import importlib
import os
import urllib.parse

from .filetype.base import BaseFileTypePlugin
from .scheme.base import BaseFileSchemePlugin

ENABLED_PLUGINS_SCHEME = [
    "jinplate.plugins.scheme.file",
    "jinplate.plugins.scheme.http",
]


ENABLED_PLUGINS_FILE_TYPE = [
    "jinplate.plugins.filetype.json",
    "jinplate.plugins.filetype.yaml",
    "jinplate.plugins.filetype.dotenv",
]


class DataLoader:
    """
    Filetype-agnostic loader for template values files
    """

    def __init__(self):
        self.scheme_plugins = DataLoader.load_scheme_plugins()
        self.file_type_plugins = DataLoader.load_file_type_plugins()

    def load(self, uri, file_type=None):
        """
        Loads the file at the given URI using ENABLED_PLUGINS_SCHEME and
        ENABLED_PLUGINS_FILE_TYPE

        :param uri: URI to the values file to load
        :param file_type: Override for file extension
        :return: The values as a dict
        """

        uri_parsed = urllib.parse.urlparse(uri)
        _, ext = os.path.splitext(uri_parsed.path)
        ext = ext.lstrip(".")

        datasource_content = None
        for plugin in self.scheme_plugins:
            if plugin.scheme_matches(uri_parsed.scheme):
                datasource_content = plugin.read(uri)
                break

        if datasource_content is None:
            raise RuntimeError(f"Unknown datasource scheme {uri_parsed.scheme}")

        datasource_parsed = None

        if file_type is None:
            for plugin in self.file_type_plugins:
                if plugin.extension_matches(ext):
                    datasource_parsed = plugin.parse(datasource_content)
        else:
            for plugin in self.file_type_plugins:
                if plugin.extension_matches(file_type):
                    datasource_parsed = plugin.parse(datasource_content)

        if datasource_parsed is None:
            raise RuntimeError(f"Unknown datasource extension {ext}")

        return datasource_parsed

    @staticmethod
    def load_scheme_plugins() -> list[BaseFileSchemePlugin]:
        """
        Loads and returns a list of plugins responsible for loading text files based on
            their uri scheme

        :return: An instance of module.FileSchemePlugin for each module in
            ENABLED_PLUGINS_SCHEME
        """

        scheme_plugins = []

        for plugin_name in ENABLED_PLUGINS_SCHEME:
            plugin_module = importlib.import_module(plugin_name)
            plugin_cls = plugin_module.FileSchemePlugin
            scheme_plugins.append(plugin_cls())

        return scheme_plugins

    @staticmethod
    def load_file_type_plugins() -> list[BaseFileTypePlugin]:
        """
        Loads and returns a list of plugins responsible for reading text files based on
            their format

        :return: An instance of module.FileTypePlugin for each module in
            ENABLED_PLUGINS_FILE_TYPE
        """

        file_type_plugins = []

        for plugin_name in ENABLED_PLUGINS_FILE_TYPE:
            plugin_module = importlib.import_module(plugin_name)
            plugin_cls = plugin_module.FileTypePlugin
            file_type_plugins.append(plugin_cls())

        return file_type_plugins



import json

from .base import BaseFileTypePlugin


class FileTypePlugin(BaseFileTypePlugin):
    """
    Plugin for reading template values from JSON files
    """

    def extension_matches(self, test_extension: str) -> bool:
        return test_extension == "json"

    def parse(self, text: str):
        return json.loads(text)



import pathlib
import urllib.parse

from .base import BaseFileSchemePlugin


class FileSchemePlugin(BaseFileSchemePlugin):
    """
    Plugin for reading template values from file uris
    """

    def scheme_matches(self, test_scheme: str) -> bool:
        return test_scheme == "file"

    def read(self, uri: str) -> str:
        uri_parsed = urllib.parse.urlparse(uri)
        return pathlib.Path(uri_parsed.path).read_text(encoding="utf-8")



import re
import urllib.parse
import urllib.request

from .base import BaseFileSchemePlugin


class FileSchemePlugin(BaseFileSchemePlugin):
    """
    Plugin for reading template values from http(s) uris
    """

    def scheme_matches(self, test_scheme: str) -> bool:
        return re.match(r"https?", test_scheme) is not None

    def read(self, uri: str) -> str:
        with urllib.request.urlopen(uri) as response:
            return response.read().decode("utf-8")



import abc


class BaseFileTypePlugin(abc.ABC):
    """
    Base class for file type plugins
    """

    @abc.abstractmethod
    def extension_matches(self, test_extension: str) -> bool:
        """
        :param test_extension: Extension to test
        :return: Whether the extension should be parsed by the plugin
        """

    @abc.abstractmethod
    def parse(self, text: str):
        """
        :param text: Text to parse
        :return: Dict of values read by the plugin
        """



import io

import dotenv

from .base import BaseFileTypePlugin


class FileTypePlugin(BaseFileTypePlugin):
    """
    Plugin for reading template values from dotenv files
    """

    def extension_matches(self, test_extension: str) -> bool:
        return test_extension == "env"

    def parse(self, text: str):
        with io.StringIO(text) as stream:
            return dotenv.dotenv_values(stream=stream)



import re

import yaml

try:
    from yaml import CLoader as YamlLoader
except ImportError:
    from yaml import Loader as YamlLoader


from .base import BaseFileTypePlugin


class FileTypePlugin(BaseFileTypePlugin):
    """
    Plugin for reading template values from YAML files
    """

    def extension_matches(self, test_extension: str) -> bool:
        return re.match(r"ya?ml", test_extension) is not None

    def parse(self, text: str):
        return yaml.load(text, Loader=YamlLoader)



#!/usr/bin/env python3

import pathlib

import click
import jinja2

from jinplate.plugins.loader import DataLoader


@click.command("jinplate")
@click.argument("template_file", type=click.Path(exists=True, dir_okay=False))
@click.argument("datasource", type=str)
@click.option("--file-type", type=str, default=None)
def jinplate_cli(template_file, datasource, file_type):
    """
    A command line renderer for jinja templates

    TEMPLATE_FILE is the path to a jinja template file to render

    DATASOURCE is the URI of a datasource supported by jinplate that contains the
    template variables
    """
    template_path = pathlib.Path(template_file).resolve()
    jenv = jinja2.Environment(loader=jinja2.FileSystemLoader(template_path.parent))
    template = jenv.get_template(template_path.name)

    dataloader = DataLoader()
    data = dataloader.load(datasource, file_type=file_type)

    print(template.render(data))


# pylint: disable=no-value-for-parameter
if __name__ == '__main__':
    jinplate_cli()



import abc


class BaseFileSchemePlugin(abc.ABC):
    """
    Base class for file scheme plugins
    """

    @abc.abstractmethod
    def scheme_matches(self, test_scheme: str) -> bool:
        """
        :param test_scheme: Scheme to test
        :return: Whether the scheme matches a file that should be fetched by the plugin
        """

    @abc.abstractmethod
    def read(self, uri: str) -> str:
        """
        :param uri: The file to fetch and read
        :return: The text content of the file read by this plugin
        """
